@@ -1,5 +1,5 @@
 package Git::Raw::Repository;
-$Git::Raw::Repository::VERSION = '0.36'; # TRIAL
+$Git::Raw::Repository::VERSION = '0.37'; # TRIAL
 use strict;
 use warnings;
 
@@ -11,7 +11,7 @@ Git::Raw::Repository - Git repository class
 
 =head1 VERSION
 
-version 0.36
+version 0.37
 
 =head1 SYNOPSIS
 
@@ -344,8 +344,8 @@ Example:
 =head2 merge_base( @objects )
 
 Find the merge base between C<@objects>. Each element in C<@objects> should be
-peelable to a C<Git::Raw::Commit> object, that is, it should be a
-C<Git::Raw::Commit> or C<Git::Raw::Reference> object, or alternatively a commit
+peelable to a L<Git::Raw::Commit> object, that is, it should be a
+L<Git::Raw::Commit> or L<Git::Raw::Reference> object, or alternatively a commit
 id or commit id prefix.
 
 =head2 merge_analysis( $reference )
@@ -412,7 +412,7 @@ Similarity metric for considering a file renamed (default is 50).
 
 =item * "target_limit"
 
-Maximum similarity sources to examine (overrides the L<"merge.renameLimit">
+Maximum similarity sources to examine (overrides the C<"merge.renameLimit">
 configuration entry) (default is 200).
 
 =back
@@ -531,7 +531,7 @@ as untracked (changing the behavior to not match core git).
 
 =item * "show_untracked_content"
 
-Include the content of untracked files. This implies L<"include_untracked">.
+Include the content of untracked files. This implies C<"include_untracked">.
 
 =item * "show_unmodified"
 
@@ -554,12 +554,12 @@ Take extra time to find minimal diff.
 =item * "a"
 
 The virtual C<"directory"> to prefix to old file names in hunk headers.
-(Default is L"a".)
+(Default is C<"a">.)
 
 =item * "b"
 
 The virtual C<"directory"> to prefix to new file names in hunk headers.
-(Default is L"b".)
+(Default is C<"b">.)
 
 =back
 
@@ -598,7 +598,7 @@ sub branch { return Git::Raw::Branch -> create(@_) }
 =head2 branches( [$type] )
 
 Retrieve a list of L<Git::Raw::Branch> objects. Possible values for C<$type>
-include L<"local">, L<"remote"> or L<"all">.
+include C<"local">, C<"remote"> or C<"all">.
 
 =head2 commit( $msg, $author, $committer, \@parents, $tree [, $update_ref ] )
 
@@ -672,6 +672,20 @@ working directory of the repository will be set to the directory.
 
 Retrieve blame information for C<$path>. Returns a C<Git::Raw::Blame> object.
 
+=head2 cherry_pick( $commit, [\%merge_opts, \%checkout_opts, $mainline] )
+
+Cherry-pick the given C<$commit>, producing changes in the index and working
+directory. See C<Git::Raw::Repository-E<gt>merge()> for valid C<%merge_opts>
+and C<%checkout_opts> values. For merge commits C<$mainline> specifies the
+parent.
+
+=head2 revert( $commit, [\%merge_opts, \%checkout_opts, $mainline] )
+
+Revert the given C<$commit>, producing changes in the index and working
+directory. See C<Git::Raw::Repository-E<gt>merge()> for valid C<%merge_opts>
+and C<%checkout_opts> values. For merge commits C<$mainline> specifies the
+parent.
+
 =head2 state( )
 
 Determine the state of the repository. One of the following values is returned:
@@ -727,7 +741,7 @@ cherry-pick, etc.
 
 =head2 message( )
 
-Retrieve the content of git's prepared message i.e. L<".git/MERGE_MSG">.
+Retrieve the content of git's prepared message i.e. C<".git/MERGE_MSG">.
 
 =head2 is_empty( )
 
@@ -743,7 +757,7 @@ Check if the repository is a shallow clone.
 
 =head2 is_head_detached( )
 
-Check if the repository's HEAD is detached, that is, it points directly to
+Check if the repository's C<HEAD> is detached, that is, it points directly to
 a commit.
 
 =head1 AUTHOR
